% Set u, f,g
u   = @u_2; 
f   = @f_2;
g   = u;

% Possibility 2
% u = x1^3.1   (u \in C^{3,0.1}, u \notin C^{3,1}):
% fun_u = @u_2;
% fun_f = @f_2;
% fun_g_D = fun_u;


% Set up discretization levels
n_max = 100;    % maximal numbers of nodes in one direction
d = 10;         % number of different stepsizes
n_space = round(logspace(1, log10(n_max),d))'; % logarithmic spacing

% Compute the corresponding values of h
h_space = 1./n_space;

% -----------------------------------------------------------------------------
% Initilization
% -----------------------------------------------------------------------------

% Initilize error-vectors
err_Linfty = zeros(d,1);
err_L2 = zeros(d,1);
err_H1Semi = zeros(d,1);

% Initilize vector for smalles and larges eigenvalues
eig_lm = zeros(d,1);
eig_sm = zeros(d,1);

% Display table header
fprintf('------------------------------------------------------------------------------------------------\n');
fprintf('   n         h      ||u-u_h||_oo     ||u-u_h||_0     ||u-u_h||_1\''    lambda_max    lambda_min \n');
fprintf('------------------------------------------------------------------------------------------------\n');

% -----------------------------------------------------------------------------
% Loop over all discretizations
% -----------------------------------------------------------------------------
for i = 1:d,

    % Set current n and h
    n = n_space(i);
    h = h_space(i);
    
    % Compute solution
    [uh,Lh] = FDM_PoissonDirichlet_UnitSquare(h, f, g, 0);
           
    % Compute the error
    x = 0:h:1; [X,Y] = meshgrid(x,x);   % set nodes for current h
    u_exact = u([X(:), Y(:)]);          % evaluates exact solution at nodes
    err = uh - u_exact;

    % Convert the error into the appropriate form
    %err_disp = reshape( err(glob_all) , n + 1 , n + 1);
    
    % Compute the L^\infty-, L^2- and H1Semi- norm of the error
    err_Linfty(i) = Linfty_Norm_Unitsquare( err );
    err_L2(i)     = L2_Norm_Unitsquare( err );
    err_H1Semi(i) = H1_Seminorm_Unitsquare( err );

    % Compute the largest and the smallest eigenvalue of system matrix
    eig_lm(i) = eigs(Lh,1,'lm');
    eig_sm(i) = eigs(Lh,1,'sm');

   % Display iteration information
   fprintf(' %4d    %8.2E    %8.2E         %8.2E        %8.2E        %8.2E      %8.2E\n',n,h,err_Linfty(i),err_L2(i),err_H1Semi(i),eig_lm(i),eig_sm(i));
end

%%

% -----------------------------------------------------------------------------
% Display the solution, the error, matrix strcture and convergence order
% -----------------------------------------------------------------------------


[uh, Lh] = FDM_PoissonDirichlet_UnitSquare(h(end), f, g, 1);

% Display the matrix sparsity structure 
figure(2); clf;
spy(Lh);
title(sprintf('Matrix sparsity structure for the 2D problem (size %d x %d)',(n+1)^2,(n+1)^2));

% Compute experimental order of convergency for the three norms
p_Linfty = eoc(h_space , err_Linfty);
p_L2     = eoc(h_space , err_L2);
p_H1Semi = eoc(h_space , err_H1Semi);

% Round the eoc, this should be the predicted theoretical order of convergence
p_Linfty_r = round(p_Linfty)
p_L2_r     = round(p_L2)
p_H1Semi_r = round(p_H1Semi)

% Display the L^\infty-, L^2- and H1Semi error and the corresponding
% discretization-levels in a loglog-plot
figure(3); clf; clear legend_str;

loglog(h_space , err_Linfty , 'b-*');
legend_str{1} = sprintf('|| u - u_h ||_{\\infty , h} (eoc = %4.2f)' , p_Linfty);
hold on;

loglog(h_space , err_L2 , 'r-*');
legend_str{2} = sprintf('|| u - u_h ||_{0 , h} (eoc = %4.2f)' , p_L2);

loglog(h_space , err_H1Semi , 'm-*');
legend_str{3} = sprintf('|| u - u_h ||_{1\'' , h} (eoc = %4.2f)' , p_H1Semi);

xlabel('mesh size h'); ylabel('error');
title('Experimental order of convergence');
legend(legend_str,'Location','NorthWest');
 
% Display the condition number
%
% Compute the condition number
cond_Lh = eig_lm ./ eig_sm;

% Compute the experimental order of convergence for the condition number
eig_p = eoc(h_space, cond_Lh);

% Display the condition number
figure(4); clf;
loglog(h_space,cond_Lh,'-*')
hold on;
xlabel('mesh size h'); ylabel('condition number');
title(sprintf('Condition number of the system matrix (eoc = %4.2f)',eig_p));
