function [uh, Lh] = FDM_PoissonDirichlet_UnitSquare(h,f,g, disp)
% solves Poisson problem with source f and Dirichlet data g on unit square
% with mesh parameter h
% via lexicographic ordering

%% Generating nodes
x = 0:h:1;
n = length(x)-1;
[X,Y] = meshgrid(x,x);
x = X(:); y = Y(:);

% lexicographic ordering
local_to_global = @(ix1,ix2) ix1 + (ix2-1) * (n+1);

%% Assembling LHS and RHS
% LHS: - discrete Laplace 
Lh = - DiscreteLaplaceMatrix_FivePointStar(n);

% RHS: evaluate f at grid nodes
fh = f([x,y]);

% Exclude lines corresponding to boundary
fh = Restrict_to_Inner(fh);

%% Take care of boundary conditions
% Determine nodes corresponding to boundary
x_south = [X(1:(n+1),1), Y(1:(n+1),1)];
id_south = local_to_global(1:(n+1), 1);

x_east = [X(n+1, 1:(n+1)); Y(n+1, 1:(n+1))]';
id_east = local_to_global(n+1, 1:(n+1));

x_north = [X(1:(n+1),n+1), Y(1:(n+1),n+1)];
id_north = local_to_global(1:(n+1), n+1);

x_west = [X(1, 1:(n+1)); Y(1, 1:(n+1))]';
id_west = local_to_global(1, 1:(n+1));

x_bnd = [x_south; x_east; x_north; x_west];
id_bnd = [id_south, id_east, id_north, id_west]';

% Evaluate g at boundary nodes
u_bnd = zeros((n+1)^2,1);
u_bnd(id_bnd) = g(x_bnd);

% Incorporate Dirichlet-Data into RHS
fh = fh + Apply_DiscreteLaplace_FivePointStar(u_bnd);

%% Compute solution
uh = Lh\fh;

% Enrich solution u with boundary data
uh = Extend_to_Boundary(uh);
uh(id_bnd) = u_bnd(id_bnd);

% Plot solution
if(nargin > 3 && disp == 1)
    figure();
    surf(X,Y, reshape(uh,n+1,n+1), 'EdgeColor','none','FaceColor','interp','FaceLighting','phong');
    xlabel('x','Fontsize',14)
    ylabel('y','Fontsize',14)
    title('Solution to Poisson problem','Fontsize',14)
end
